/*  ERRSPEC.GSS:  Collection of procedures used to compute error spectra
    implied by a theoretical model and corresponding data.  Also generates
    Watson's Relative Mean Squared Approximation Error statsitics.

|
|	Adoptation of GAUSS code written by Y. Wen.


*/

/*  The primary procedure contained in this file is ERRSPEC.PRC.  It is
    called using the following syntax:

    {specerr, specmod, fit} = errspec(modspec,datspec,nord,wght,per1,per2);

Inputs:

    modspec and datspec are spectrum matricies corresponding to the model
    and data;  nord is the number of ordinates over which the spectra are
    to be computed;  wght is a (nvars x 1) vector of relative weights to be
    assigned to the individual series;  per1 and per2 give lower and
    upper bounds for frequency range used to compute Watson's RMSqAE
    stats (e.g., per1 = 6, per2 = 32 would compute the statistics over
    the 6 to 32 period range;  per1 = 2, per2 = 10000000 would compute
    the stats over all frequencies).

Outputs:

    specerr = error spectrum
    specmod = cross-spec (model, data)
    fit = vector of Watson's RMSqAE statistics

*/

/*  STEP 1:  compile specme.prc, which calculates the error spectral
             density matrix for a given frequency
*/

proc (2) = specme(spmod,spdat,wght);

/* Procedure for Calculating Error Spectral Density Matrix
   Input:
         spmod = model spectrum
         spdat = data spectrum
         wght = weight (to weight trace)

   Output:
         sperr = error spectrum
         spmd = cross-spec (model, data)
*/

local k, u, s, v, a, dif, b, w, atil, g, spmd, sperr;

@ -- (1) Decompose spmod=a*a',  where a is nxk, and k = rank(spmod) -- @
 k=rank(spmod);
 {u,s,v}=svd2(spmod);
 a=u[.,1:k]*(sqrt(s[1:k,1:k]));

@ -- (2) Calculate Matrix Square Root of SPDATA, spdat = b*b' @
 {u,s,v}=svd2(spdat);
 b=u*(sqrt(s));

@ -- (3) Find G, such that:
         sxy=A*G'; SPdat = g*g' + See, where See is psd @

 w=diagrv(eye(rows(a)),wght);


 atil=b'*w*a;
 {u,s,v}=svd2(atil);
 g=b*u[.,1:k]*v';
@ -- (4) Calculate Spectrum of Error -- @
 spmd=a*g';
 sperr=spmod+spdat-spmd-spmd';

retp(sperr,spmd);
endp;


/*  Compile errspec.prc  */

proc (3) = errspec(modspec,datspec,nord,wght,per1,per2);

local nt1, nt2, sverr, svmd, svgamt, spmod, spdat, j, sperr, spmd, fit,
      step, wvec, perd, per1per2, ns, k, ess, datss, modss, spgam, nvars;

nvars = rows(wght);
nt1 = nvars;
nt1 = nt1*(nt1+1)/2;
sverr = zeros(nt1,nord);
nt2 = nvars;
nt2 = nt2*nt2;
svmd = zeros(nt2,nord);
svgamt = zeros(nt2,nord);

j=1; do while j<= nord;
   spmod=xpnd(modspec[.,j]);     @ Model Spectrum -- "un-veched" @
   spdat=xpnd(datspec[.,j]);     @ Data Spectrum -- "un-veched" @
   {sperr,spmd}=specme(spmod,spdat,wght);
   sverr[.,j]=vech(sperr);     @ Error Spectrum @
   svmd[.,j]=vec(spmd);        @ Model-Data Cross Spectrum @
   spgam=spmd*(inv(spdat));    @ Gamma Matrix -- relating Model to data @
   svgamt[.,j]=vec(spgam');    @ SGGAMT Contains the VEC of the Transpose
                                 of the GAMMA matrix @
 j=j+1; endo;

/*  Now compute Watson's statistics  */


fit = zeros(nvars,1);
step = pi/nord;
wvec = seqa(step,step,nord);
perd = 2*pi*ones(nord,1)./wvec;  @ Vector with Periods @
per1per2 = (perd .>= per1) .and (perd .<= per2);
ns=1; do while ns <= nvars;
   k = ns*(ns+1)/2;
     ess = real(sumc(sverr[k,.]'.*per1per2));
     datss=real(sumc(datspec[k,.]'.*per1per2));
     modss=real(sumc(modspec[k,1]'.*per1per2));
     fit[ns,.]=ess/datss;
ns=ns+1; endo;

retp(sverr,svmd,fit);
endp;



