@
|  MODSPEC.GSS:  COLLECTION OF PROCEDURES USED TO COMPUTE SPECTRA
|                  ASSOCIATED WITH A THEORETICAL MODEL.
|
|
|	Adoptation of GAUSS code written by Y. Wen.
|
|      The model is assumed to produce a state-space representation of the
|        form
|            s(t) = m*s(t-1) + g*e(t)    (law of motion for the full set
|                                         of variables)
|
|            v(t) = h*s(t)               (collection of observables)
|
|
|      This file contains three procedures:
|
|           spec = modspec(m,g,h,nord);
|
|       where: m is lag matrix
|              g is the matrix multiplying the errors
|              h is the observability matrix
|              nord is the number of ordinates (power of 2)
|       output is spec (k x nord). Each column is the vech of the
|                spectral density for the relevant frequency. Spectrum
|                is not calculated at zero.
|                      *******************************
|           sv = sspec(m,g,h,w);
|
|       where: m,g,h as above
|              w is required frequency
|       output is sv, the spectrum of v at frequency w. Not normalized by 2*pi.
|                      ********************************
|           svmod = specdif(lspec,w);
|
|       where lspec is the spectral density matrix
|             w is required frequency
|       output is specdif, spectral density matrix of first differences.
|
@
proc sspec(m,g,h,w);

@
|    Procedure for Calculating Spectral Density Matrix for Variables
|   in the State Space Model:
@
    local im, sm;
    let im = 0+1i;
    sm=eye(rows(m))-m*exp(-w*im);
    sm=inv(sm);
    sm=sm*g;
    retp(h*sm*sm'h');
endp;

proc specdif(lspec,w);
    local im, a;
    let im = 0+1i;
    a = (1-exp(-w*im))*eye(rows(lspec));
    retp(a*lspec*a');
endp;


proc modspec(m,g,h,nord);
    local nt, svmod, step, wvec, j, sv, svdif;

    nt = rows(h);
    svmod = zeros(nt*(nt+1)/2,nord);
    step = pi/nord;
    wvec = seqa(step,step,nord);
    j = 0;
    do while j < nord;
        j=j+1;
        sv = sspec(m,g,h,wvec[j,.]);     @ Spectrum of Levels @
        svdif = specdif(sv,wvec[j,.]);   @ Spectrum of First Differences @
        svmod[.,j] = vech(svdif);
    endo;

retp(svmod);
endp;
