@  kalman.prc
|
|  procedure to do a kalman filter and smoothing for the system:
|       Zt+1 = FZt + vt+1,   Evv' = Q
|       Yt = H'Zt
@
/*	To the best of our knowledge, this code was written by B.F. Ingram */

/*
Inputs:
	y, Txnvars matrix of data
	f,h,q: see above
	smooth, 0,1 variable. if zero, do not run smoother; if 1, run smoother (back out unobservables)

Output:
	eps, smoothed system innovations
	p, vcv matricies of observations of zs
	z, smoothed zs
	lkwght, log likelihood value

Format:
	       {eps,p,z,lkwght} = kalman(y,f,h,q,smooth);
*/

proc  (4) = kalman(y,f,h,q,smooth);
    local iq,phhhp,zttm1,rowsff,rowsy,nvars,ilike,pttm1,i,vareps,matsum,
            epsdraw,hphinv,wyres,yres,ptt,ztt,like,z,p,ztcapt,ptcapt,jt,
            temp,ptp1capt,ztp1capt,j,meaneps,kttm1,mttm1,sprod,T,sttm1;

    rowsff = rows(f);
    rowsy = rows(y);
    nvars = cols(y);

@
|   These are the starting values. We start the recursion at the
|   unconditional mean.
@
    zttm1 = zeros(rowsff,1);
    ilike = zeros(rowsy,1);
    wyres = zeros(rowsy,rowsff);
    sttm1 = eye(rowsff);
    z = zttm1'~zttm1';
    pttm1 = reshape(inv(eye(rowsff^2) - (f.*.f))*vec(q),rowsff,rowsff);

    p = pttm1~pttm1~pttm1;
    i=0;
    iq = eye(rowsff);
@
|   Start the Kalman filter
@
    do while i < rowsy;
        i=i+1;

        hphinv = pinv(h'pttm1*h);  
@
|   Check to make sure hphinv is positive definite. If not, stop.
@
       if ismiss(hphinv);

            retp("hphinv not p.d.",i,0,0); goto endd;
        endif;

        phhhp = pttm1*h*hphinv;  

        yres = y[i,.]' - h'zttm1;
        wyres[i,.] = (f'h*hphinv*yres)';

        sttm1 = sttm1|f'(iq - h*hphinv*h'pttm1);
@
|       pttm1 = f*(I - pttm1*h*inv(h'pttm1*h)*h')*pttm1*f' + q
@

        ptt   = pttm1 - phhhp*h'pttm1;

        pttm1 = f*ptt*f' + q;

@
|   p contains pt/t and pt+1/t, which are each rowsff x rowsff
@

        p = p|ptt~pttm1~pttm1;
        ztt = zttm1 + phhhp*yres;
        zttm1 = f*ztt;
@
|   z contains zt/t and zt+1/t, which are rowsff x 1
@
        z = z|ztt'~zttm1';
        ilike[i,.] = 0.5*ln(det(hphinv)) - 0.5*yres'hphinv*yres;
    endo;
@
|   delete the first rows of p and z, which were temp values
@
    p = p[rowsff+1:rows(p),.];
    sttm1 = sttm1[rowsff+1:rows(sttm1),.];
    z = z[2:rows(z),.];
    T = rows(z);
    like = -rowsy*0.5*nvars*ln(2*pi) + sumc(ilike);
@
|   only run this part if you want the smoothed values. Will only run
|   if smooth = 1. If smooth = 0, will goto endif
@
    epsdraw=0;
    if smooth;
        ztcapt = z[T,1:rowsff]';
        ptcapt = p[rowsff*(T-1)+1:rowsff*T,1:rowsff];
        p[rowsff*(T-1)+1:rowsff*T,2*rowsff+1:3*rowsff] = ptcapt;
        z[T,rowsff+1:2*rowsff] = ztcapt';
        epsdraw = zeros(T,rowsff);
        i = T;
        mttm1 = wyres[T,.]';
        kttm1 = 0;
        do while i > 1;
            i=i-1;
@
|   reload the kalman filtered values
@
            ptt = p[rowsff*(i-1)+1:rowsff*i,1:rowsff];
            ztt = z[i,1:rowsff]';

            ztcapt = ztt + ptt*mttm1;

            epsdraw[i+1,.] = z[i+1,rowsff+1:2*rowsff]-(f*ztcapt)';

            z[i,rowsff+1:2*rowsff] = ztcapt';
            mttm1 = sttm1[rowsff*i+1:rowsff*(i+1),1:rowsff]*mttm1
                                                + wyres[i,.]';
        endo;
    endif;

 endd:

retp(epsdraw,p,z,like);
endp;


