/*  spec_var.prc:  procedures to compute spectra using var. */

/*     The data are assumed to be prefiltered (e.g., detrended). */

/*   This file contains:
|
|           spec.prc, var.prc, spvar.prc
*/

/*  	Format for var.prc:  {beta,s} = var(y,nlags)
	Inputs:  	y:	nobs x nvars matrix of variables
            		nlags:	number of lags in VAR
	Output:		beta: 	VAR parameters
			s:	vcv matrix for errors
*/

/*	Format for spvar:	ss = spvar{beta,s,w)
	Inputs:		beta,s:	output from var.prc
			w:	frequency value
	Output:		ss:	spectral density matrix at given frequency
*/

/*	Format for spec.prc:	{sp, fr} = spec(y,nlags,nord);
	Inputs:		y, nlags as above.
			nord is number of ordinates over which to compute spectra (must be power of 2)
	Output:		sp:	nvars x nord spectral density ordinates
			fr:	frequencies (measured as cycles per period)
*/


proc (2) = var(y,nlags);
    local nobs, ndep, x, i, beta, e;
    nobs = (rows(y));
    ndep = nobs - nlags;
    x=ones(ndep,1);


    i=1;
    do while i<=nlags;
        x = x~y[nlags+1-i:nobs-i,.];
        i=i+1;
    endo;

    y=y[nlags+1:nobs,.];
    beta = invpd(x'x)*x'y;
    e = y-x*beta;
    retp(beta[2:rows(beta),.]',e'e/ndep);
endp;


proc spvar(b,seps,w);
    local im, z, z1, sm, smi, ss, nvar, nlags, ii, n1;

    nvar = rows(seps);       @ Number of Variables in VAR @
    nlags = cols(b)/(nvar);

@ -- Calculate Spectrum of S --- @
    let im = 0+1i;
    z = exp(-w*im);
    z1 = 1;
    sm = eye(nvar);
    ii=1;
    do while ii <= nlags;
        n1 = nvar*(ii-1);
        z1 = z*z1;
        sm = sm-z1*b[.,n1+1:n1+nvar];
        ii=ii+1;
    endo;
    smi = inv(sm);
    ss = smi*seps*smi';
@ ss=ss/(2*pi);  @

retp(ss);
endp;


proc (2) = spec(y,nlags,nord);
    local beta, seps, nt, svdat, step, wvec, j, ss, ssdif, c, i, k;

@  Estimate VAR  @
    {beta, seps} = var(y,nlags); 

@  Build spectra  @
    nt = rows(seps);
    svdat = zeros(nt*(nt+1)/2,nord);
    step = pi/nord;
    wvec = seqa(step,step,nord);
    j=1;
    do while j<=nord;
        ss = spvar(beta,seps,wvec[j,.]);  @ Spectrum of VAR @
        svdat[.,j] = vech(ss);
        j=j+1;
    endo;

    c = zeros(nord,nt);
    i=1; do while i<=nt;
	k = i*(i+1)/2;
	c[.,i] = real(svdat[k,.]')/(2*pi);
    i=i+1; endo;

    wvec = wvec/(2*pi);

retp(c,wvec);
endp;



